> {-# LANGUAGE UnicodeSyntax, UndecidableInstances #-}

> module FreeMonad
> where

Free monad of a functor.

> data Free f a = Var a | Com (f (Free f a))
>
> instance (Show a, Show (f (Free f a))) => Show (Free f a) where
>   show (Var x) = show x
>   show (Com x) = show x

> eval' :: (Functor f) => (a -> b) -> (f b -> b) -> (Free f a -> b)
> eval' var _com (Var x) = var x
> eval' var  com (Com x) = com (fmap (eval' var com) x)

Simpler version, closer to a fold.

> eval :: (Functor f) => (f b -> b) -> (Free f b -> b)
> eval _com (Var x) = x
> eval  com (Com x) = com (fmap (eval com) x)

We have |eval' v c = eval c . fmap v|.

> join :: (Functor f) => Free f (Free f a) -> Free f a
> join (Var x) = x
> join (Com x) = Com (fmap join x)

Shorter: |join = eval Com|.

> instance (Functor f) => Functor (Free f) where
>   fmap f (Var x) = Var (f x)
>   fmap f (Com x) = Com (fmap (fmap f) x)

> instance (Functor f) => Monad (Free f) where
>   return  = Var
>   m >>= k = join (fmap k m)

> embed :: (Functor f) => f a -> Free f a
> embed = Com . fmap Var

Converting between F-algebras and M-algebras.

> up :: (Functor f) => (f a -> a) -> (Free f a -> a)
> up a = eval a

> down :: (Functor f) => (Free f a -> a) -> (f a -> a)
> down a = a . embed
