> import InitialAlgebra
> import FinalCoalgebra
> import FreeMonad

Syntax functor.

> data S a = One | Plus (a, a) | FBy (Integer, a) | Nat
>   deriving (Show)

NB. |FBy|, short for ``followed by'', embeds semantics into syntax.

> instance Functor S where
>  fmap _f One           = One
>  fmap  f (Plus (p, q)) = Plus (f p, f q)
>  fmap  f (FBy (a, p))  = FBy (a, f p)
>  fmap _f Nat           = Nat

> ex :: Mu S
> ex = In (Plus (In One, In One))

Behaviour functor.

> data B a = Cons (Integer, a)
>   deriving (Show)
>
> instance Functor B where
>   fmap f (Cons (n, s)) = Cons (n, f s)

Stream equations.

> rho :: S (B a) -> B (Free S a)
> rho One
>   = Cons (1, Com One)
> rho (Plus (Cons (a1, p1), Cons (a2, p2)))
>   = Cons (a1 + a2, Com (Plus (Var p1, Var p2)))
> rho (FBy (m, Cons (n, s)))
>   = Cons (m, Com (FBy (n, Var s)))
> rho Nat
>   = Cons (0, Com (Plus (Com Nat, Com One)))

Distributive law.

> lambda :: Free S (B a) -> B (Free S a)
> lambda = eval' (fmap Var) (fmap join . rho)

> colift :: (a -> B a) -> (Free S a -> B (Free S a))
> colift   c = lambda . fmap c

> lift :: (Free S a -> a) -> (Free S (B a) -> B a)
> lift a = fmap a . lambda

Semantics of closed terms (denotational and operational).

> dsem, osem :: Mu S -> Nu B
> dsem = fold   (down (unfold (colift out    )))
> osem = unfold (fold (down   (lift   (up In))))

Semantics of open terms (only operational).

> sem :: Free S (Nu B) -> Nu B
> -- ad-hoc definition: |sem = unfold (eval' (fmap Var . out) (fmap join . rho))|
> sem = unfold (colift out)

Semantic definitions of stream constants/operators.

> one :: Nu B
> one = sem (Com One)

> plus :: (Nu B, Nu B) -> Nu B
> plus (s, t) = sem (Com (Plus (Var s, Var t)))

> nat :: Nu B
> nat = sem (Com Nat)