> import FreeMonad
> import CofreeComonad

Syntax functor.

> data S a = One | Plus (a, a) | Id a | Even a | Nat
>   deriving (Show)
>
> instance Functor S where
>  fmap _f One           = One
>  fmap  f (Plus (p, q)) = Plus (f p, f q)
>  fmap  f (Id p)        = Id (f p)
>  fmap  f (Even p)      = Even (f p)
>  fmap _f Nat           = Nat

Behaviour functor.

> data B a = Cons (Integer, a)
>   deriving (Show)
>
> instance Functor B where
>   fmap f (Cons (n, s)) = Cons (n, f s)

Stream equations.

> {-
> rho :: S (Cofree B a) -> B (Free S a)
> rho (One) 
>   = Cons (1, Com One)
> rho (Plus (Root _ (Cons (a, Root s' _)), Root _ (Cons (b, Root t' _))))
>   = Cons (a + b, Com (Plus (Var s', Var t')))
> rho (Id (Root _ (Cons (a, Root s _))))
>   = Cons (a, Com (Id (Var s)))
> rho (Even (Root _ (Cons (a, Root _ (Cons (_, Root s _))))))
>   = Cons (a, Com (Even (Var s)))

Corecursively defined!

> rho :: S (Cofree B (Free S a)) -> Cofree B (Free S a)
> rho One
>   = Root (Com One) (Cons (1, rho One))
> rho (Plus (Root p (Cons (a, s')), Root q (Cons (b, t'))))
>   = Root (Com (Plus (p, q))) (Cons (a + b, rho (Plus (s', t'))))
> rho (Id (Root p (Cons (a, s))))
>   = Root (Com (Id p)) (Cons (a, rho (Id s)))
> rho (Even (Root p (Cons (a, Root _ (Cons (_, s))))))
>   = Root (Com (Even p)) (Cons (a, rho (Even s)))
> rho Nat
>   = Root (Com Nat) (Cons (0, rho (Plus (rho Nat, rho One))))

> lambda :: Free S (Cofree B a) -> Cofree B (Free S a)
> lambda = eval' (fmap Var) rho

Try: |lambda (Com Nat)| or |lambda (Com (Id (Com Nat)))|.

> -}

Deriving a distributive law.

> lambda :: Free S (Cofree B a) -> Cofree B (Free S a)
> lambda = eval' (fmap Var) (trace (Com . fmap extract) rho)

> rho :: S (Cofree B (Free S a)) -> B (S (Cofree B (Free S a)))
> rho One
>   = Cons (1, One)
> rho (Plus (Root p (Cons (a, s)), Root q (Cons (b, t))))
>   = Cons (a + b, Plus (s, t))
> rho Nat
> --  = Cons (0, Plus (Root (Com One) undefined, Root (Com Nat) undefined)) -- ???
>   = Cons (0, Plus (lambda (Com One), lambda (Com Nat))) -- ???
> rho (Even (Root p (Cons (a, Root _ (Cons (_, s))))))
>   = Cons (a, Even s)

