> {-# LANGUAGE NPlusKPatterns #-}
> module Stream
> where
> import Prelude hiding (map, repeat, sum, zip)

> data Stream α  =  SCons { shead :: α, stail :: Stream α }
>
> infixr 5 ≺
> (≺)    ::  α -> Stream α -> Stream α
> a ≺ s  =   SCons a s

> repeat :: a -> Stream a
> repeat a = s where s = SCons a s

> map :: (a -> b) -> (Stream a -> Stream b)
> map f ~(SCons a s) = SCons (f a) (map f s)

> zip :: (a -> b -> c) -> (Stream a -> Stream b -> Stream c)
> zip g ~(SCons a s) ~(SCons b t) = SCons (g a b) (zip g s t)

> first             ::  Int -> Stream α -> [α]
> first 0       _s  =   []
> first (n + 1)  s  =   shead s : first n (stail s)
> first _  _        =   error "first: negative argument"

> showStream      :: (Show a) => Int -> Stream a -> IO ()
> showStream l s  =  putStrLn (showStreamS l s "")

> showStreamS :: (Show a) => Int -> Stream a -> ShowS
> showStreamS l s  =  showString "<# " . showl (first l s)
>   where showl []        =  showString "... #>"
>         showl (a : as)  =  shows a . showString ", " . showl as

> len  ::  Int
> len  =   16

> instance (Show a) => Show (Stream a) where
>   showsPrec _d  =  showStreamS len  -- HACK
> instance (Eq a) => Eq (Stream a) where
>   s == t  =  first len s == first len t  -- HACK
> instance (Ord a) => Ord (Stream a) where
>   s <= t  =  first len s <= first len t  -- HACK

`Generic' instances for |Functor| and numeric classes.

> instance Functor Stream where
>   fmap  =  map

> instance (Num a) => Num (Stream a) where
>   (+)          =  zip (+)
>   (-)          =  zip (-)
>   (*)          =  zip (*)
>   negate       =  map negate
>   abs          =  map abs
>   signum       =  map signum
>   fromInteger  =  repeat . fromInteger
